USE StockData
GO

/****************************************************

	Window functions, including ROW_NUMBER(), allow
	intra-query shifting or numbering functions

****************************************************/

/*
	Get the amount of the last dividend of the year for each stock ticker.
	Use ROW_NUMBER() to number, in reverse order, the dividend dates
*/

SELECT t.ticker
     , t.dividend_date
     , t.dividend_amount
FROM (
	SELECT ticker, dividend_date, dividend_amount
		, ROW_NUMBER() OVER (PARTITION BY ticker ORDER BY dividend_date DESC) AS RowNum
	FROM dbo.Dividends
	WHERE YEAR(dividend_date) = 2024
) t
WHERE t.RowNum = 1


