USE StockData
GO

/******************************************************************

	CTEs provide a way for abstracting queries to simplify
	logic therein. They work sort of like ad-hoc views for
	the duration of a single query.

*******************************************************************/


/*
	Get the min and max dividends for a particular year, and use
	that to calculate the percentage of variance
*/

-- Don't forget the semicolon before the CTE!
;WITH dividendCte AS (
	SELECT ticker
		, MIN(dividend_amount) AS minDividend
		, MAX(dividend_amount) AS maxDividend
	FROM dbo.Dividends
	WHERE YEAR(dividend_date) = 2024
	GROUP BY ticker
)

SELECT dividendCte.ticker
	, dividendCte.minDividend
	, dividendCte.maxDividend
	, (dividendCte.maxDividend - dividendCte.minDividend) / dividendCte.minDividend AS MovementPct
FROM dividendCte
ORDER BY (dividendCte.maxDividend - dividendCte.minDividend) / dividendCte.minDividend DESC




-- Use two different CTEs to compare year-over-year dividend performance
;WITH cte2023 AS (
	SELECT ticker
		, sum(dividend_amount) AS dividend_amount
	FROM dbo.Dividends
	WHERE YEAR(dividend_date) = 2023
	GROUP BY ticker
)
, cte2024 AS (
	SELECT ticker
		, sum(dividend_amount) AS dividend_amount
	FROM dbo.Dividends
	WHERE YEAR(dividend_date) = 2024
	GROUP BY ticker
)

SELECT c3.ticker
	, c3.dividend_amount AS PY_Dividend
	, c4.dividend_amount AS CY_Dividend
	, (c4.dividend_amount - c3.dividend_amount) / c3.dividend_amount AS Improvement_PCT
FROM cte2023 AS c3
INNER JOIN cte2024 AS c4
	ON c4.ticker = c3.ticker
ORDER BY Improvement_PCT DESC



