USE StockData
GO

/*
	Show a potentially unexpected behavior when using NOT IN with NULLs
*/


-- Which equities do not have any dividends in the staged data?
SELECT * 
FROM dbo.Stocks t
WHERE t.ticker NOT IN (SELECT ticker FROM stg.Dividends)

-- But that can't be correct, since there are more Stocks than 
--	staged dividends.
SELECT COUNT(DISTINCT ticker) AS TickerCount FROM dbo.Stocks
SELECT COUNT(DISTINCT ticker) AS DividendTickerCount FROM stg.Dividends


-- Looking more closely, there's at least one NULL ticker in the staged dividend data
--	(side note: Always validate your data in staging!)
SELECT * 
FROM stg.Dividends
WHERE ticker IS NULL 


-- For extra insurance against this behavior, eliminate nulls
SELECT * 
FROM dbo.Stocks t
WHERE t.ticker NOT IN (SELECT ticker FROM stg.Dividends WHERE ticker IS NOT NULL)



-- Even better, use NOT EXISTS instead of NOT IN for better performance on large data sets
SELECT * 
FROM dbo.Stocks t
WHERE NOT EXISTS (SELECT 1 FROM stg.Dividends h WHERE h.ticker = t.ticker)
