USE StockData
GO


/*
	Use COALESCE instead of ISNULL to allow for multi-step null checks without nesting ISNULL functions
*/



;WITH cteTickers AS (
	SELECT ticker
		, YEAR([date]) AS [Year]
		, adj_close_price AS [ClosePrice]
		, ROW_NUMBER() OVER (PARTITION BY ticker, YEAR([date]) ORDER BY [date] DESC) AS RowNum
	FROM dbo.StockHistory
	WHERE YEAR([date]) >= 2020
	AND ticker IN ('MSFT', 'META', 'AAPL', 'AMZN', 'NFLX', 'GOOG', 'GOOGL')
)

, cteYear AS (
	-- Remember CTEs? You can also use multiple CTEs and reference earlier-declared CTEs.
	SELECT cteTickers.ticker
         , cteTickers.Year
         , cteTickers.ClosePrice         
	FROM cteTickers
	WHERE cteTickers.RowNum = 1
)

SELECT COALESCE(cte20.ticker, cte21.ticker, cte22.ticker, cte23.ticker, cte24.ticker) as Ticker
	, cte20.ClosePrice AS [2020]
	, cte21.ClosePrice AS [2021]
	, cte22.ClosePrice AS [2022]
	, cte23.ClosePrice AS [2023]
	, cte24.ClosePrice AS [2024]
FROM cteYear cte20
FULL OUTER JOIN cteYear cte21
	ON cte21.ticker = cte20.ticker
	AND cte20.Year = 2020
	AND cte21.Year = 2021
FULL OUTER JOIN cteYear cte22
	ON cte22.ticker = COALESCE(cte21.ticker, cte20.ticker)
	AND cte22.Year = 2022
FULL OUTER JOIN cteYear cte23
	ON cte23.ticker = COALESCE(cte22.ticker, cte21.ticker, cte20.ticker)
	AND cte23.Year = 2023
FULL OUTER JOIN cteYear cte24
	ON cte24.ticker = COALESCE(cte23.ticker, cte22.ticker, cte21.ticker, cte20.ticker)
	AND cte24.Year = 2024
WHERE COALESCE(cte20.Year, 2020) = 2020
	AND COALESCE(cte21.Year, 2021) = 2021
	AND COALESCE(cte22.Year, 2022) = 2022
	AND COALESCE(cte23.Year, 2023) = 2023
	AND COALESCE(cte24.Year, 2024) = 2024

