USE StockData


;WITH cteTbl AS (
	SELECT SCHEMA_NAME(tbl.schema_id)   AS SchemaName
		 , tbl.name                     AS TableName
		 , p.name                       AS ExtendedPropertyName
		 , CAST(p.value AS SQL_VARIANT) AS ExtendedPropertyValue
	FROM sys.tables                        AS tbl
	INNER JOIN sys.extended_properties AS p
		ON p.major_id = tbl.object_id
		AND p.minor_id = 0
		AND p.class = 1
)

SELECT cteDesc.SchemaName
     , cteDesc.TableName
     , cteDesc.ExtendedPropertyValue AS TableDescription
	 , cteSrcTyp.ExtendedPropertyValue AS TableSourceType
	 , cteSrc.ExtendedPropertyValue AS TableSource
FROM cteTbl cteDesc
INNER JOIN cteTbl cteSrcTyp
	ON cteSrcTyp.SchemaName = cteDesc.SchemaName
	AND cteSrcTyp.TableName = cteDesc.TableName
	AND cteDesc.ExtendedPropertyName = 'TableDescription'
	AND cteSrcTyp.ExtendedPropertyName = 'TableSourceType'
INNER JOIN cteTbl AS cteSrc
	ON cteSrc.SchemaName = cteDesc.SchemaName
	AND cteSrc.TableName = cteDesc.TableName
	AND ctesrc.ExtendedPropertyName = 'TableSource'


/*
	Show column-level extended properties
*/

;WITH cteCol AS (
	SELECT S.name   AS SchemaName
		 , O.name   AS TableName
		 , c.name AS ColumnName
		 , c.column_id AS ColumnID
		 , EP.name AS PropertyName
		 , EP.value AS PropertyValue
	FROM sys.extended_properties   EP
	INNER JOIN sys.all_objects O
		ON EP.major_id = O.object_id
	INNER JOIN sys.schemas     S
		ON O.schema_id = S.schema_id
	INNER JOIN sys.columns     AS c
		ON EP.major_id = c.object_id
		AND EP.minor_id = c.column_id
)


SELECT colSrc.SchemaName
     , colSrc.TableName
	 , colSrc.ColumnName
	 , colSrc.ColumnID
     , colSrc.PropertyValue AS ColumnSource
	 , colDesc.PropertyValue AS ColumnDescription

FROM cteCol colSrc
INNER JOIN cteCol colDesc
	ON colDesc.SchemaName = colSrc.SchemaName
	AND colDesc.TableName = colSrc.TableName
	AND colDesc.ColumnName = colSrc.ColumnName
	AND colSrc.PropertyName = 'ColumnSource'
	AND colDesc.PropertyName = 'ColumnDescription'

ORDER BY colSrc.SchemaName
	, colSrc.TableName
	, colSrc.ColumnID